# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from functools import cached_property


class BaseProfile(str, Enum):
    """
    Profile parent. Use it if you need your own profiles.

    All pycgmes objects requiring a Profile are actually asking for a `BaseProfile`. As
    Enum with fields cannot be inherited or composed, just create your own CustomProfile without
    trying to extend Profile. It will work.
    """

    @cached_property
    def long_name(self) -> str:
        """Return the long name of the profile."""
        return self.value


class Profile(BaseProfile):
    """
    Enum containing all CGMES profiles and their export priority.
    """

    # DI= "DiagramLayout" # Not too sure about that one
    DL = "DiagramLayout"
    DY = "Dynamics"
    EQ = "Equipment"
    EQBD = "EquipmentBoundary"  # Not too sure about that one
    GL = "GeographicalLocation"
    OP = "Operation"
    SC = "ShortCircuit"
    SSH = "SteadyStateHypothesis"
    SV = "StateVariables"
    TP = "Topology"
    TPBD = "TopologyBoundary"  # Not too sure about that one



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ACDCTerminal import ACDCTerminal


@dataclass(config=DataclassConfig)
class Terminal(ACDCTerminal):
    """
    An AC electrical connection point to a piece of conducting equipment. Terminals are connected at physical connection
      points called connectivity nodes.

    TopologicalNode: The topological node associated with the terminal.   This can be used as an alternative to the
      connectivity node path to topological node, thus making it unnecessary to model connectivity
      nodes in some cases.   Note that the if connectivity nodes are in the model, this association
      would probably not be used as an input specification.
    ConductingEquipment: The conducting equipment of the terminal.  Conducting equipment have  terminals that may be
      connected to other conducting equipment terminals via connectivity nodes or topological
      nodes.
    ConnectivityNode: The connectivity node to which this terminal connects with zero impedance.
    ConverterDCSides: All converters` DC sides linked to this point of common coupling terminal.
    AuxiliaryEquipment: The auxiliary equipment connected to the terminal.
    RegulatingControl: The controls regulating this terminal.
    phases: Represents the normal network phasing condition. If the attribute is missing, three phases (ABC) shall be
      assumed, except for terminals of grounding classes (specializations of EarthFaultCompensator,
      GroundDisconnector, and Ground) which will be assumed to be N. Therefore, phase code ABCN is
      explicitly declared when needed, e.g. for star point grounding equipment. The phase code on terminals
      connecting same ConnectivityNode or same TopologicalNode as well as for equipment between two
      terminals shall be consistent.
    TransformerEnd: All transformer ends connected at this terminal.
    TieFlow: The control area tie flows to which this terminal associates.
    HasSecondMutualCoupling: Mutual couplings with the branch associated as the first branch.
    HasFirstMutualCoupling: Mutual couplings associated with the branch as the first branch.
    SvPowerFlow: The power flow state variable associated with the terminal.
    RemoteInputSignal: Input signal coming from this terminal.
    """

    TopologicalNode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.TP,
        ],
    )

    ConductingEquipment: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQBD,
            Profile.EQ,
            Profile.DY,
        ],
    )

    ConnectivityNode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQBD,
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # ConverterDCSides : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # AuxiliaryEquipment : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # RegulatingControl : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    phases: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # TransformerEnd : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    # *Association not used*
    # Type M:0..2 in CIM  # pylint: disable-next=line-too-long
    # TieFlow : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # HasSecondMutualCoupling : list = Field(default_factory=list, in_profiles = [Profile.SC, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # HasFirstMutualCoupling : list = Field(default_factory=list, in_profiles = [Profile.SC, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # SvPowerFlow : Optional[str] = Field(default=None, in_profiles = [Profile.SV, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # RemoteInputSignal : list = Field(default_factory=list, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.TP,
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TapChanger import TapChanger


@dataclass(config=DataclassConfig)
class RatioTapChanger(TapChanger):
    """
    A tap changer that changes the voltage ratio impacting the voltage magnitude but not the phase angle across the
      transformer.  Angle sign convention (general): Positive value indicates a positive phase shift from the
      winding where the tap is located to the other winding (for a two-winding transformer).

    stepVoltageIncrement: Tap step increment, in per cent of rated voltage of the power transformer end, per step
      position. When the increment is negative, the voltage decreases when the tap step
      increases.
    RatioTapChangerTable: The tap ratio table for this ratio  tap changer.
    TransformerEnd: Transformer end to which this ratio tap changer belongs.
    """

    stepVoltageIncrement: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    RatioTapChangerTable: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    TransformerEnd: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SSH,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEST2A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type ST2A model. Some static systems use both current and voltage sources (generator terminal
      quantities) to comprise the power source.  The regulator controls the exciter output through controlled
      saturation of the power transformer components.  These compound-source rectifier excitation systems are
      designated type ST2A and are represented by ExcIEEEST2A. Reference: IEEE 421.5-2005, 7.2.

    ka: Voltage regulator gain (KA) (> 0).  Typical value = 120.
    ta: Voltage regulator time constant (TA) (> 0).  Typical value = 0,15.
    vrmax: Maximum voltage regulator outputs (VRMAX) (> 0).  Typical value = 1.
    vrmin: Minimum voltage regulator outputs (VRMIN) (<= 0).  Typical value = 0.
    ke: Exciter constant related to self-excited field (KE).  Typical value = 1.
    te: Exciter time constant, integration rate associated with exciter control (TE) (> 0).  Typical value = 0,5.
    kf: Excitation control system stabilizer gains (KF) (>= 0).  Typical value = 0,05.
    tf: Excitation control system stabilizer time constant (TF) (>= 0).  Typical value = 1.
    kp: Potential circuit gain coefficient (KP) (>= 0).  Typical value = 4,88.
    ki: Potential circuit gain coefficient (KI) (>= 0).  Typical value = 8.
    kc: Rectifier loading factor proportional to commutating reactance (KC) (>= 0). Typical value = 1,82.
    efdmax: Maximum field voltage (EFDMax) (>= 0).  Typical value = 99.
    uelin: UEL input (UELin). true = HV gate false = add to error signal. Typical value = true.
    """

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efdmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uelin: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class Status(Base):
    """
    Current status information relevant to an entity.

    value: Status value at `dateTime`; prior status changes may have been kept in instances of activity records
      associated with the object to which this status applies.
    dateTime: Date and time for which status `value` applies.
    remark: Pertinent information regarding the current `value`, as free form text.
    reason: Reason code or explanation for why an object went to the current status `value`.
    """

    value: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    dateTime: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    remark: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    reason: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.GL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEAC7B(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type AC7B model. The model represents excitation systems which consist of an AC alternator with
      either stationary or rotating rectifiers to produce the DC field requirements. It is an upgrade to earlier AC
      excitation systems, which replace only the controls but retain the AC alternator and diode rectifier bridge.
      Reference: IEEE 421.5-2005, 6.7. Note, however, that in IEEE 421.5-2005, the [1 / sTE] block is shown as [1 /
      (1 + sTE)], which is incorrect.

    kpr: Voltage regulator proportional gain (KPR) (> 0 if ExcIEEEAC7B.kir = 0).  Typical value = 4,24.
    kir: Voltage regulator integral gain (KIR) (>= 0).  Typical value = 4,24.
    kdr: Voltage regulator derivative gain (KDR) (>= 0).  Typical value = 0.
    tdr: Lag time constant (TDR) (>= 0).  Typical value = 0.
    vrmax: Maximum voltage regulator output (VRMAX) (> 0).  Typical value = 5,79.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0).  Typical value = -5,79.
    kpa: Voltage regulator proportional gain (KPA) (> 0 if ExcIEEEAC7B.kia = 0).  Typical value = 65,36.
    kia: Voltage regulator integral gain (KIA) (>= 0).  Typical value = 59,69.
    vamax: Maximum voltage regulator output (VAMAX) (> 0).  Typical value = 1.
    vamin: Minimum voltage regulator output (VAMIN) (< 0).  Typical value = -0,95.
    kp: Potential circuit gain coefficient (KP) (> 0).  Typical value = 4,96.
    kl: Exciter field voltage lower limit parameter (KL).  Typical value = 10.
    te: Exciter time constant, integration rate associated with exciter control (TE) (> 0).  Typical value = 1,1.
    vfemax: Exciter field current limit reference (VFEMAX).  Typical value = 6,9.
    vemin: Minimum exciter voltage output (VEMIN) (<= 0).  Typical value = 0.
    ke: Exciter constant related to self-excited field (KE).  Typical value = 1.
    kc: Rectifier loading factor proportional to commutating reactance (KC) (>= 0). Typical value = 0,18.
    kd: Demagnetizing factor, a function of exciter alternator reactances (KD) (>= 0).  Typical value = 0,02.
    kf1: Excitation control system stabilizer gain (KF1) (>= 0).  Typical value = 0,212.
    kf2: Excitation control system stabilizer gain (KF2) (>= 0).  Typical value = 0.
    kf3: Excitation control system stabilizer gain (KF3) (>= 0).  Typical value = 0.
    tf: Excitation control system stabilizer time constant (TF) (> 0).  Typical value = 1.
    ve1: Exciter alternator output voltages back of commutating reactance at which saturation is defined (VE1) (> 0).
      Typical value = 6,3.
    seve1: Exciter saturation function value at the corresponding exciter voltage, VE1, back of commutating reactance
      (SE[VE1]) (>= 0).  Typical value = 0,44.
    ve2: Exciter alternator output voltages back of commutating reactance at which saturation is defined (VE2) (> 0).
      Typical value = 3,02.
    seve2: Exciter saturation function value at the corresponding exciter voltage, VE2, back of commutating reactance
      (SE[VE2]) (>= 0).  Typical value = 0,075.
    """

    kpr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kir: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kdr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tdr: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpa: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kia: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vfemax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vemin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    seve2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class ConnectivityNode(IdentifiedObject):
    """
    Connectivity nodes are points where terminals of AC conducting equipment are connected together with zero impedance.

    TopologicalNode: The topological node to which this connectivity node is assigned.  May depend on the current state
      of switches in the network.
    BoundaryPoint: The boundary point associated with the connectivity node.
    Terminals: Terminals interconnected with zero impedance at a this connectivity node.
    ConnectivityNodeContainer: Container of this connectivity node.
    """

    TopologicalNode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.TP,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # BoundaryPoint : Optional[str] = Field(default=None, in_profiles = [Profile.EQBD, Profile.EQ, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # Terminals : list = Field(default_factory=list, in_profiles = [Profile.EQBD, Profile.EQ, ])

    ConnectivityNodeContainer: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQBD,
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.TP,
            Profile.EQBD,
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .VoltageAdjusterDynamics import VoltageAdjusterDynamics


@dataclass(config=DataclassConfig)
class VAdjIEEE(VoltageAdjusterDynamics):
    """
    IEEE voltage adjuster which is used to represent the voltage adjuster in either a power factor or VAr control
      system. Reference: IEEE 421.5-2005, 11.1.

    vadjf: Set high to provide a continuous raise or lower (VADJF).
    adjslew: Rate at which output of adjuster changes (ADJ_SLEW).  Unit = s / PU.  Typical value = 300.
    vadjmax: Maximum output of the adjuster (VADJMAX) (> VAdjIEEE.vadjmin).  Typical value = 1,1.
    vadjmin: Minimum output of the adjuster (VADJMIN) (< VAdjIEEE.vadjmax).  Typical value = 0,9.
    taon: Time that adjuster pulses are on (TAON) (>= 0).  Typical value = 0,1.
    taoff: Time that adjuster pulses are off (TAOFF) (>= 0).  Typical value = 0,5.
    """

    vadjf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    adjslew: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vadjmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vadjmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    taon: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    taoff: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class StreetDetail(Base):
    """
    Street details, in the context of address.

    number: Designator of the specific location on the street.
    name: Name of the street.
    suffix: Suffix to the street name. For example: North, South, East, West.
    prefix: Prefix to the street name. For example: North, South, East, West.
    type: Type of street. Examples include: street, circle, boulevard, avenue, road, drive, etc.
    code: (if applicable) Utilities often make use of external reference systems, such as those of the town-planner`s
      department or surveyor general`s mapping system, that allocate global reference codes to streets.
    buildingName: (if applicable) In certain cases the physical location of the place of interest does not have a direct
      point of entry from the street, but may be located inside a larger structure such as a building,
      complex, office block, apartment, etc.
    suiteNumber: Number of the apartment or suite.
    addressGeneral: First line of a free form address or some additional address information (for example a mail stop).
    addressGeneral2: (if applicable) Second line of a free form address.
    addressGeneral3: (if applicable) Third line of a free form address.
    withinTownLimits: True if this street is within the legal geographical boundaries of the specified town (default).
    floorIdentification: The identification by name or number, expressed as text, of the floor in the building as part
      of this address.
    """

    number: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    name: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    suffix: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    prefix: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    type: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    code: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    buildingName: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    suiteNumber: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    addressGeneral: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    addressGeneral2: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    addressGeneral3: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    withinTownLimits: bool = Field(
        default=False,
        in_profiles=[
            Profile.GL,
        ],
    )

    floorIdentification: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.GL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovGAST2(TurbineGovernorDynamics):
    """
    Gas turbine.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    w: Governor gain (1/droop) on turbine rating (W).
    x: Governor lead time constant (X) (>= 0).
    y: Governor lag time constant (Y) (> 0).
    z: Governor mode (Z). 1 = droop 0 = isochronous.
    etd: Turbine and exhaust delay (Etd) (>= 0).
    tcd: Compressor discharge time constant (Tcd) (>= 0).
    trate: Turbine rating (Trate).  Unit = MW.
    t: Fuel control time constant (T) (>= 0).
    tmax: Maximum turbine limit (Tmax) (> GovGAST2.tmin).
    tmin: Minimum turbine limit (Tmin) (< GovGAST2.tmax).
    ecr: Combustion reaction time delay (Ecr) (>= 0).
    k3: Ratio of fuel adjustment (K3).
    a: Valve positioner (A).
    b: Valve positioner (B).
    c: Valve positioner (C).
    tf: Fuel system time constant (Tf) (>= 0).
    kf: Fuel system feedback (Kf).
    k5: Gain of radiation shield (K5).
    k4: Gain of radiation shield (K4).
    t3: Radiation shield time constant (T3) (>= 0).
    t4: Thermocouple time constant (T4) (>= 0).
    tt: Temperature controller integration rate (Tt) (>= 0).
    t5: Temperature control time constant (T5) (>= 0).
    af1: Exhaust temperature parameter (Af1).  Unit = PU temperature.  Based on temperature in degrees C.
    bf1: (Bf1).  Bf1 = E(1 - W) where E (speed sensitivity coefficient) is 0,55 to 0,65 x Tr.  Unit = PU temperature.
      Based on temperature in degrees C.
    af2: Coefficient equal to 0,5(1-speed) (Af2).
    bf2: Turbine torque coefficient Khhv (depends on heating value of fuel stream in combustion chamber) (Bf2).
    cf2: Coefficient defining fuel flow where power output is 0% (Cf2).  Synchronous but no output.  Typically 0,23 x
      Khhv (23% fuel flow).
    tr: Rated temperature (Tr).  Unit = [SYMBOL REMOVED]C depending on parameters Af1 and Bf1.
    k6: Minimum fuel flow (K6).
    tc: Temperature control (Tc).  Unit = [SYMBOL REMOVED]F or [SYMBOL REMOVED]C depending on parameters Af1 and Bf1.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    w: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    x: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    y: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    z: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    etd: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tcd: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    trate: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ecr: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    b: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    c: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tt: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    af1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    bf1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    af2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    bf2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    cf2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tr: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovSteamEU(TurbineGovernorDynamics):
    """
    Simplified boiler and steam turbine with PID governor.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    tp: Power transducer time constant (Tp) (>= 0).  Typical value = 0,07.
    ke: Gain of the power controller (Ke).  Typical value = 0,65.
    tip: Integral time constant of the power controller (Tip) (>= 0).  Typical value = 2.
    tdp: Derivative time constant of the power controller (Tdp) (>= 0).  Typical value = 0.
    tfp: Time constant of the power controller (Tfp) (>= 0).  Typical value = 0.
    tf: Frequency transducer time constant (Tf) (>= 0).  Typical value = 0.
    kfcor: Gain of the frequency corrector (Kfcor).  Typical value = 20.
    db1: Deadband of the frequency corrector (db1).  Typical value = 0.
    wfmax: Upper limit for frequency correction (Wfmax) (> GovSteamEU.wfmin).  Typical value = 0,05.
    wfmin: Lower limit for frequency correction (Wfmin) (< GovSteamEU.wfmax).  Typical value = -0,05.
    pmax: Maximal active power of the turbine (Pmax).  Typical value = 1.
    ten: Electro hydraulic transducer (Ten) (>= 0).  Typical value = 0,1.
    tw: Speed transducer time constant (Tw) (>= 0).  Typical value = 0,02.
    komegacor: Gain of the speed governor (Kwcor).  Typical value = 20.
    db2: Deadband of the speed governor (db2).  Typical value = 0,0004.
    wwmax: Upper limit for the speed governor (Wwmax) (> GovSteamEU.wwmin).  Typical value = 0,1.
    wwmin: Lower limit for the speed governor frequency correction (Wwmin) (< GovSteamEU.wwmax).  Typical value = -1.
    wmax1: Emergency speed control lower limit (wmax1).  Typical value = 1,025.
    wmax2: Emergency speed control upper limit (wmax2).  Typical value = 1,05.
    tvhp: Control valves servo time constant (Tvhp) (>= 0).  Typical value = 0,1.
    cho: Control valves rate opening limit (Cho).  Unit = PU / s.  Typical value = 0,17.
    chc: Control valves rate closing limit (Chc).  Unit = PU / s.  Typical value = -3,3.
    hhpmax: Maximum control valve position (Hhpmax).  Typical value = 1.
    tvip: Intercept valves servo time constant (Tvip) (>= 0).  Typical value = 0,15.
    cio: Intercept valves rate opening limit (Cio).  Typical value = 0,123.
    cic: Intercept valves rate closing limit (Cic).  Typical value = -2,2.
    simx: Intercept valves transfer limit (Simx).  Typical value = 0,425.
    thp: High pressure (HP) time constant of the turbine (Thp) (>= 0).  Typical value = 0,31.
    trh: Reheater  time constant of the turbine (Trh) (>= 0).  Typical value = 8.
    tlp: Low pressure (LP) time constant of the turbine (Tlp) (>= 0).  Typical value = 0,45.
    prhmax: Maximum low pressure limit (Prhmax).  Typical value = 1,4.
    khp: Fraction of total turbine output generated by HP part (Khp).  Typical value = 0,277.
    klp: Fraction of total turbine output generated by HP part (Klp).  Typical value = 0,723.
    tb: Boiler time constant (Tb) (>= 0).  Typical value = 100.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tip: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tdp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tfp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kfcor: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    db1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    wfmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    wfmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ten: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    komegacor: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    db2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    wwmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    wwmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    wmax1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    wmax2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tvhp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    cho: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    chc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    hhpmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tvip: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    cio: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    cic: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    simx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    trh: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tlp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    prhmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    khp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    klp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class Curve(IdentifiedObject):
    """
    A multi-purpose curve or functional relationship between an independent variable (X-axis) and dependent (Y-axis)
      variables.

    curveStyle: The style or shape of the curve.
    xUnit: The X-axis units of measure.
    y1Unit: The Y1-axis units of measure.
    y2Unit: The Y2-axis units of measure.
    CurveDatas: The point data values that define this curve.
    """

    curveStyle: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    xUnit: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    y1Unit: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    y2Unit: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # CurveDatas : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class SvInjection(Base):
    """
    The SvInjection reports the calculated bus injection minus the sum of the terminal flows. The terminal flow is
      positive out from the bus (load sign convention) and bus injection has positive flow into the bus. SvInjection
      may have the remainder after state estimation or slack after power flow calculation.

    pInjection: The active power mismatch between calculated injection and initial injection.  Positive sign means
      injection into the TopologicalNode (bus).
    qInjection: The reactive power mismatch between calculated injection and initial injection.  Positive sign means
      injection into the TopologicalNode (bus).
    TopologicalNode: The topological node associated with the flow injection state variable.
    """

    pInjection: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SV,
        ],
    )

    qInjection: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SV,
        ],
    )

    TopologicalNode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SV,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.SV,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .WindTurbineType1or2Dynamics import WindTurbineType1or2Dynamics


@dataclass(config=DataclassConfig)
class WindTurbineType1or2IEC(WindTurbineType1or2Dynamics):
    """
    Parent class supporting relationships to IEC wind turbines type 1 and type 2 including their control models.
      Generator model for wind turbine of IEC type 1 or type 2 is a standard asynchronous generator model.
      Reference: IEC 61400-27-1:2015, 5.5.2 and 5.5.3.

    WindMechIEC: Wind mechanical model associated with this wind generator type 1 or type 2 model.
    WindProtectionIEC: Wind turbune protection model associated with this wind generator type 1 or type 2 model.
    """

    WindMechIEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    WindProtectionIEC: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssRQB(PowerSystemStabilizerDynamics):
    """
    Power system stabilizer type RQB. This power system stabilizer is intended to be used together with excitation
      system type ExcRQB, which is primarily used in nuclear or thermal generating units.

    ki2: Speed input gain (Ki2). Typical value = 3,43.
    ki3: Electrical power input gain (Ki3). Typical value = -11,45.
    ki4: Mechanical power input gain (Ki4). Typical value = 11,86.
    t4m: Input time constant (T4M) (>= 0). Typical value = 5.
    tomd: Speed delay (TOMD) (>= 0). Typical value = 0,02.
    tomsl: Speed time constant (TOMSL) (>= 0). Typical value = 0,04.
    t4mom: Speed time constant (T4MOM) (>= 0). Typical value = 1,27.
    sibv: Speed deadband (SIBV). Typical value = 0,006.
    kdpm: Lead lag gain (KDPM). Typical value = 0,185.
    t4f: Lead lag time constant (T4F) (>= 0). Typical value = 0,045.
    """

    ki2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4m: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tomd: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tomsl: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4mom: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    sibv: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kdpm: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4f: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindPlantReactiveControlIEC(IdentifiedObject):
    """
    Simplified plant voltage and reactive power control model for use with type 3 and type 4 wind turbine models.
      Reference: IEC 61400-27-1:2015, Annex D.

    WindDynamicsLookupTable: The wind dynamics lookup table associated with this voltage and reactive power wind plant
      model.
    WindPlantIEC: Wind plant reactive control model associated with this wind plant.
    dxrefmax: Maximum positive ramp rate for wind turbine reactive power/voltage reference (dxrefmax) (>
      WindPlantReactiveControlIEC.dxrefmin). It is a project-dependent parameter.
    dxrefmin: Maximum negative ramp rate for wind turbine reactive power/voltage reference (dxrefmin) (<
      WindPlantReactiveControlIEC.dxrefmax). It is a project-dependent parameter.
    kiwpx: Plant Q controller integral gain (KIWPx). It is a project-dependent parameter.
    kiwpxmax: Maximum reactive power/voltage reference from integration (KIWPxmax) (>
      WindPlantReactiveControlIEC.kiwpxmin). It is a project-dependent parameter.
    kiwpxmin: Minimum reactive power/voltage reference from integration (KIWPxmin) (<
      WindPlantReactiveControlIEC.kiwpxmax). It is a project-dependent parameter.
    kpwpx: Plant Q controller proportional gain (KPWPx). It is a project-dependent parameter.
    kwpqref: Reactive power reference gain (KWPqref). It is a project-dependent parameter.
    kwpqu: Plant voltage control droop (KWPqu). It is a project-dependent parameter.
    tuqfilt: Filter time constant for voltage-dependent reactive power (Tuqfilt) (>= 0). It is a project-dependent
      parameter.
    twppfiltq: Filter time constant for active power measurement (TWPpfiltq) (>= 0). It is a project-dependent
      parameter.
    twpqfiltq: Filter time constant for reactive power measurement (TWPqfiltq) (>= 0). It is a project-dependent
      parameter.
    twpufiltq: Filter time constant for voltage measurement (TWPufiltq) (>= 0). It is a project-dependent parameter.
    txft: Lead time constant in reference value transfer function (Txft) (>= 0). It is a project-dependent parameter.
    txfv: Lag time constant in reference value transfer function (Txfv) (>= 0). It is a project-dependent parameter.
    uwpqdip: Voltage threshold for UVRT detection in Q control (uWPqdip). It is a project-dependent parameter.
    windPlantQcontrolModesType: Reactive power/voltage controller mode (MWPqmode). It is a case-dependent parameter.
    xrefmax: Maximum xWTref (qWTref or delta uWTref) request from the plant controller (xrefmax) (>
      WindPlantReactiveControlIEC.xrefmin). It is a case-dependent parameter.
    xrefmin: Minimum xWTref (qWTref or delta uWTref) request from the plant controller (xrefmin) (<
      WindPlantReactiveControlIEC.xrefmax). It is a project-dependent parameter.
    """

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # WindDynamicsLookupTable : list = Field(default_factory=list, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:1 in CIM  # pylint: disable-next=line-too-long
    # WindPlantIEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    dxrefmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dxrefmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kiwpx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kiwpxmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kiwpxmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpwpx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kwpqref: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kwpqu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tuqfilt: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    twppfiltq: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    twpqfiltq: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    twpufiltq: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    txft: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    txfv: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uwpqdip: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    windPlantQcontrolModesType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    xrefmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xrefmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemResource import PowerSystemResource


@dataclass(config=DataclassConfig)
class CogenerationPlant(PowerSystemResource):
    """
    A set of thermal generating units for the production of electrical energy and process steam (usually from the output
      of the steam turbines). The steam sendout is typically used for industrial purposes or for municipal heating
      and cooling.

    ThermalGeneratingUnits: A thermal generating unit may be a member of a cogeneration plant.
    """

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # ThermalGeneratingUnits : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .DiscontinuousExcitationControlDynamics import DiscontinuousExcitationControlDynamics


@dataclass(config=DataclassConfig)
class DiscExcContIEEEDEC2A(DiscontinuousExcitationControlDynamics):
    """
    IEEE type DEC2A model for discontinuous excitation control. This system provides transient excitation boosting via
      an open-loop control as initiated by a trigger signal generated remotely. Reference: IEEE 421.5-2005 12.3.

    vk: Discontinuous controller input reference (VK).
    td1: Discontinuous controller time constant (TD1) (>= 0).
    td2: Discontinuous controller washout time constant (TD2) (>= 0).
    vdmin: Limiter (VDMIN) (< DiscExcContIEEEDEC2A.vdmax).
    vdmax: Limiter (VDMAX) (> DiscExcContIEEEDEC2A.vdmin).
    """

    vk: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    td1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    td2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vdmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vdmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PFVArControllerType1Dynamics import PFVArControllerType1Dynamics


@dataclass(config=DataclassConfig)
class PFVArType1IEEEPFController(PFVArControllerType1Dynamics):
    """
    IEEE PF controller type 1 which operates by moving the voltage reference directly. Reference: IEEE 421.5-2005, 11.2.

    ovex: Overexcitation Flag (OVEX) true = overexcited false = underexcited.
    tpfc: PF controller time delay (TPFC) (>= 0).  Typical value = 5.
    vitmin: Minimum machine terminal current needed to enable pf/var controller (VITMIN).
    vpf: Synchronous machine power factor (VPF).
    vpfcbw: PF controller deadband (VPFC_BW).  Typical value = 0,05.
    vpfref: PF controller reference (VPFREF).
    vvtmax: Maximum machine terminal voltage needed for pf/var controller to be enabled (VVTMAX) (>
      PFVArType1IEEEPFController.vvtmin).
    vvtmin: Minimum machine terminal voltage needed to enable pf/var controller (VVTMIN) (<
      PFVArType1IEEEPFController.vvtmax).
    """

    ovex: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpfc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vitmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vpf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vpfcbw: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vpfref: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvtmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vvtmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .OverexcitationLimiterDynamics import OverexcitationLimiterDynamics


@dataclass(config=DataclassConfig)
class OverexcLimX1(OverexcitationLimiterDynamics):
    """
    Field voltage over excitation limiter.

    efdrated: Rated field voltage (EFDRATED).  Typical value = 1,05.
    efd1: Low voltage point on the inverse time characteristic (EFD1).  Typical value = 1,1.
    t1: Time to trip the exciter at the low voltage point on the inverse time characteristic (TIME1) (>= 0).  Typical
      value = 120.
    efd2: Mid voltage point on the inverse time characteristic (EFD2).  Typical value = 1,2.
    t2: Time to trip the exciter at the mid voltage point on the inverse time characteristic (TIME2) (>= 0).  Typical
      value = 40.
    efd3: High voltage point on the inverse time characteristic (EFD3).  Typical value = 1,5.
    t3: Time to trip the exciter at the high voltage point on the inverse time characteristic (TIME3) (>= 0).  Typical
      value = 15.
    efddes: Desired field voltage (EFDDES).  Typical value = 0,9.
    kmx: Gain (KMX).  Typical value = 0,01.
    vlow: Low voltage limit (VLOW) (> 0).
    """

    efdrated: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efd3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efddes: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kmx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vlow: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class Pss5(PowerSystemStabilizerDynamics):
    """
    Detailed Italian PSS.

    kpe: Electric power input gain (KPE).  Typical value = 0,3.
    kf: Frequency/shaft speed input gain (KF).  Typical value = 5.
    isfreq: Selector for frequency/shaft speed input (isFreq). true = speed (same meaning as InputSignaKind.rotorSpeed)
      false = frequency (same meaning as InputSignalKind.busFrequency). Typical value = true (same meaning
      as InputSignalKind.rotorSpeed).
    kpss: PSS gain (KPSS).  Typical value = 1.
    ctw2: Selector for second washout enabling (CTW2). true = second washout filter is bypassed false = second washout
      filter in use. Typical value = true.
    tw1: First washout (TW1) (>= 0).  Typical value = 3,5.
    tw2: Second washout (TW2) (>= 0).  Typical value = 0.
    tl1: Lead/lag time constant (TL1) (>= 0).  Typical value = 0.
    tl2: Lead/lag time constant (TL2) (>= 0).  If = 0, both blocks are bypassed.  Typical value = 0.
    tl3: Lead/lag time constant (TL3) (>= 0).  Typical value = 0.
    tl4: Lead/lag time constant (TL4) (>= 0).  If = 0, both blocks are bypassed.  Typical value = 0.
    vsmn: Stabilizer output maximum limit (VSMN).  Typical value = -0,1.
    vsmx: Stabilizer output minimum limit (VSMX).  Typical value = 0,1.
    tpe: Electric power filter time constant (TPE) (>= 0).  Typical value = 0,05.
    pmin: Minimum power PSS enabling (Pmin).  Typical value = 0,25.
    deadband: Stabilizer output deadband (DEADBAND).  Typical value = 0.
    vadat: Signal selector (VadAtt). true = closed (generator power is greater than Pmin) false = open (Pe is smaller
      than Pmin). Typical value = true.
    """

    kpe: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    isfreq: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpss: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ctw2: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tl4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsmn: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vsmx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpe: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    deadband: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vadat: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class PssELIN2(PowerSystemStabilizerDynamics):
    """
    Power system stabilizer typically associated with ExcELIN2 (though PssIEEE2B or Pss2B can also be used).

    ts1: Time constant (Ts1) (>= 0).  Typical value = 0.
    ts2: Time constant (Ts2) (>= 0).  Typical value = 1.
    ts3: Time constant (Ts3) (>= 0).  Typical value = 1.
    ts4: Time constant (Ts4) (>= 0).  Typical value = 0,1.
    ts5: Time constant (Ts5) (>= 0).  Typical value = 0.
    ts6: Time constant (Ts6) (>= 0).  Typical value = 1.
    ks1: Gain (Ks1).  Typical value = 1.
    ks2: Gain (Ks2).  Typical value = 0,1.
    ppss: Coefficient (p_PSS) (>= 0 and <= 4).  Typical value = 0,1.
    apss: Coefficient (a_PSS).  Typical value = 0,1.
    psslim: PSS limiter (psslim).  Typical value = 0,1.
    """

    ts1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ppss: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    apss: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    psslim: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovHydro1(TurbineGovernorDynamics):
    """
    Basic hydro turbine governor.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    rperm: Permanent droop (R) (> 0).  Typical value = 0,04.
    rtemp: Temporary droop (r) (> GovHydro1.rperm).  Typical value = 0,3.
    tr: Washout time constant (Tr) (> 0).  Typical value = 5.
    tf: Filter time constant (Tf) (> 0).  Typical value = 0,05.
    tg: Gate servo time constant (Tg) (> 0).  Typical value = 0,5.
    velm: Maximum gate velocity (Vlem) (> 0).  Typical value = 0,2.
    gmax: Maximum gate opening (Gmax) (> 0 and > GovHydro.gmin).  Typical value = 1.
    gmin: Minimum gate opening (Gmin) (>= 0 and < GovHydro1.gmax).  Typical value = 0.
    tw: Water inertia time constant (Tw) (> 0).  Typical value = 1.
    at: Turbine gain (At) (> 0).  Typical value = 1,2.
    dturb: Turbine damping factor (Dturb) (>= 0).  Typical value = 0,5.
    qnl: No-load flow at nominal head (qnl) (>= 0).  Typical value = 0,08.
    hdam: Turbine nominal head (hdam).  Typical value = 1.
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rperm: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    rtemp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tr: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tg: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    velm: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    gmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tw: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    at: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dturb: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    qnl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    hdam: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcREXS(ExcitationSystemDynamics):
    """
    General purpose rotating excitation system.  This model can be used to represent a wide range of excitation systems
      whose DC power source is an AC or DC generator. It encompasses IEEE type AC1, AC2, DC1, and DC2 excitation
      system models.

    e1: Field voltage value 1 (E1).  Typical value = 3.
    e2: Field voltage value 2 (E2).  Typical value = 4.
    fbf: Rate feedback signal flag (fbf). Typical value = fieldCurrent.
    flimf: Limit type flag (Flimf).  Typical value = 0.
    kc: Rectifier regulation factor (Kc).  Typical value = 0,05.
    kd: Exciter regulation factor (Kd).  Typical value = 2.
    ke: Exciter field proportional constant (Ke).  Typical value = 1.
    kefd: Field voltage feedback gain (Kefd).  Typical value = 0.
    kf: Rate feedback gain (Kf) (>= 0).  Typical value = 0,05.
    kh: Field voltage controller feedback gain (Kh).  Typical value = 0.
    kii: Field current regulator integral gain (Kii).  Typical value = 0.
    kip: Field current regulator proportional gain (Kip).  Typical value = 1.
    ks: Coefficient to allow different usage of the model-speed coefficient (Ks).  Typical value = 0.
    kvi: Voltage regulator integral gain (Kvi).  Typical value = 0.
    kvp: Voltage regulator proportional gain (Kvp).  Typical value = 2800.
    kvphz: V/Hz limiter gain (Kvphz).  Typical value = 0.
    nvphz: Pickup speed of V/Hz limiter (Nvphz).  Typical value = 0.
    se1: Saturation factor at E1 (Se1).  Typical value = 0,0001.
    se2: Saturation factor at E2 (Se2).  Typical value = 0,001.
    ta: Voltage regulator time constant (Ta) (>= 0).  If = 0, block is bypassed.  Typical value = 0,01.
    tb1: Lag time constant (Tb1) (>= 0).  If = 0, block is bypassed.  Typical value = 0.
    tb2: Lag time constant (Tb2) (>= 0).  If = 0, block is bypassed.  Typical value = 0.
    tc1: Lead time constant (Tc1) (>= 0).  Typical value = 0.
    tc2: Lead time constant (Tc2) (>= 0).  Typical value = 0.
    te: Exciter field time constant (Te) (> 0).  Typical value = 1,2.
    tf: Rate feedback time constant (Tf) (>= 0).  If = 0, the feedback path is not used.  Typical value = 1.
    tf1: Feedback lead time constant (Tf1) (>= 0).  Typical value = 0.
    tf2: Feedback lag time constant (Tf2) (>= 0).  If = 0, block is bypassed.  Typical value = 0.
    tp: Field current bridge time constant (Tp) (>= 0).  Typical value = 0.
    vcmax: Maximum compounding voltage (Vcmax).  Typical value = 0.
    vfmax: Maximum exciter field current (Vfmax) (> ExcREXS.vfmin).  Typical value = 47.
    vfmin: Minimum exciter field current (Vfmin) (< ExcREXS.vfmax).  Typical value = -20.
    vimax: Voltage regulator input limit (Vimax).  Typical value = 0,1.
    vrmax: Maximum controller output (Vrmax) (> ExcREXS.vrmin).  Typical value = 47.
    vrmin: Minimum controller output (Vrmin) (< ExcREXS.vrmax).  Typical value = -20.
    xc: Exciter compounding reactance (Xc).  Typical value = 0.
    """

    e1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    e2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fbf: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    flimf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ke: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kefd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kh: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kii: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kip: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kvi: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kvp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kvphz: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    nvphz: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    se1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    se2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    te: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tf2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tp: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vcmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vfmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vfmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class CoordinateSystem(IdentifiedObject):
    """
    Coordinate reference system.

    crsUrn: A Uniform Resource Name (URN) for the coordinate reference system (crs) used to define
      `Location.PositionPoints`. An example would be the European Petroleum Survey Group (EPSG) code for a
      coordinate reference system, defined in URN under the Open Geospatial Consortium (OGC) namespace as:
      urn:ogc:def:crs:EPSG::XXXX, where XXXX is an EPSG code (a full list of codes can be found at the EPSG
      Registry web site http://www.epsg-registry.org/). To define the coordinate system as being WGS84
      (latitude, longitude) using an EPSG OGC, this attribute would be urn:ogc:def:crs:EPSG::4236. A profile
      should limit this code to a set of allowed URNs agreed to by all sending and receiving parties.
    Locations: All locations described with position points in this coordinate system.
    """

    crsUrn: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # Locations : list = Field(default_factory=list, in_profiles = [Profile.GL, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.GL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class SvTapStep(Base):
    """
    State variable for transformer tap step.

    position: The floating point tap position.   This is not the tap ratio, but rather the tap step position as defined
      by the related tap changer model and normally is constrained to be within the range of minimum and
      maximum tap positions.
    TapChanger: The tap changer associated with the tap step state.
    """

    position: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SV,
        ],
    )

    TapChanger: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SV,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.SV,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class UnitSymbol(Base):
    """
    The derived units defined for usage in the CIM. In some cases, the derived unit is equal to an SI unit. Whenever
      possible, the standard derived symbol is used instead of the formula for the derived unit. For example, the
      unit symbol Farad is defined as "F" instead of "CPerV". In cases where a standard symbol does not exist for a
      derived unit, the formula for the unit is used as the unit symbol. For example, density does not have a
      standard symbol and so it is represented as "kgPerm3". With the exception of the "kg", which is an SI unit,
      the unit symbols do not contain multipliers and therefore represent the base derived unit to which a
      multiplier can be applied as a whole.  Every unit symbol is treated as an unparseable text as if it were a
      single-letter symbol. The meaning of each unit symbol is defined by the accompanying descriptive text and not
      by the text contents of the unit symbol. To allow the widest possible range of serializations without
      requiring special character handling, several substitutions are made which deviate from the format described
      in IEC 80000-1. The division symbol "/" is replaced by the letters "Per". Exponents are written in plain text
      after the unit as "m3" instead of being formatted as "m" with a superscript of 3  or introducing a symbol as
      in "m^3". The degree symbol "[SYMBOL REMOVED]" is replaced with the letters "deg". Any clarification of the
      meaning for a substitution is included in the description for the unit symbol. Non-SI units are included in
      list of unit symbols to allow sources of data to be correctly labelled with their non-SI units (for example, a
      GPS sensor that is reporting numbers that represent feet instead of meters). This allows software to use the
      unit symbol information correctly convert and scale the raw data of those sources into SI-based units.  The
      integer values are used for harmonization with IEC 61850.

    """

    # No attributes defined for this class.

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DL,
            Profile.EQBD,
            Profile.EQ,
            Profile.SC,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class DCNode(IdentifiedObject):
    """
    DC nodes are points where terminals of DC conducting equipment are connected together with zero impedance.

    DCTopologicalNode: The DC topological node to which this DC connectivity node is assigned.  May depend on the
      current state of switches in the network.
    DCTerminals: DC base terminals interconnected with zero impedance at a this DC connectivity node.
    DCEquipmentContainer: The DC container for the DC nodes.
    """

    DCTopologicalNode: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.TP,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # DCTerminals : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    DCEquipmentContainer: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.TP,
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class MutualCoupling(IdentifiedObject):
    """
    This class represents the zero sequence line mutual coupling.

    b0ch: Zero sequence mutual coupling shunt (charging) susceptance, uniformly distributed, of the entire line section.
    distance11: Distance to the start of the coupled region from the first line`s terminal having sequence number equal
      to 1.
    distance12: Distance to the end of the coupled region from the first line`s terminal with sequence number equal to
      1.
    distance21: Distance to the start of coupled region from the second line`s terminal with sequence number equal to 1.
    distance22: Distance to the end of coupled region from the second line`s terminal with sequence number equal to 1.
    g0ch: Zero sequence mutual coupling shunt (charging) conductance, uniformly distributed, of the entire line section.
    r0: Zero sequence branch-to-branch mutual impedance coupling, resistance.
    x0: Zero sequence branch-to-branch mutual impedance coupling, reactance.
    Second_Terminal: The starting terminal for the calculation of distances along the second branch of the mutual
      coupling.
    First_Terminal: The starting terminal for the calculation of distances along the first branch of the mutual
      coupling.  Normally MutualCoupling would only be used for terminals of AC line segments.  The
      first and second terminals of a mutual coupling should point to different AC line segments.
    """

    b0ch: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    distance11: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    distance12: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    distance21: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    distance22: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    g0ch: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    r0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    x0: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SC,
        ],
    )

    Second_Terminal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SC,
        ],
    )

    First_Terminal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SC,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.SC,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindProtectionIEC(IdentifiedObject):
    """
    The grid protection model includes protection against over- and under-voltage, and against over- and under-
      frequency. Reference: IEC 61400-27-1:2015, 5.6.6.

    WindDynamicsLookupTable: The wind dynamics lookup table associated with this grid protection model.
    dfimax: Maximum rate of change of frequency (dFmax). It is a type-dependent parameter.
    fover: Wind turbine over frequency protection activation threshold (fover). It is a project-dependent parameter.
    funder: Wind turbine under frequency protection activation threshold (funder). It is a project-dependent parameter.
    mzc: Zero crossing measurement mode (Mzc).  It is a type-dependent parameter.  true = WT protection system uses zero
      crossings to detect frequency (1 in the IEC model) false = WT protection system does not use zero
      crossings to detect frequency (0 in the IEC model).
    tfma: Time interval of moving average window (TfMA) (>= 0).  It is a type-dependent parameter.
    uover: Wind turbine over voltage protection activation threshold (uover). It is a project-dependent parameter.
    uunder: Wind turbine under voltage protection activation threshold (uunder). It is a project-dependent parameter.
    WindTurbineType3or4IEC: Wind generator type 3 or type 4 model with which this wind turbine protection model is
      associated.
    WindTurbineType1or2IEC: Wind generator type 1 or type 2 model with which this wind turbine protection model is
      associated.
    """

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # WindDynamicsLookupTable : list = Field(default_factory=list, in_profiles = [Profile.DY, ])

    dfimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    fover: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    funder: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    mzc: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    tfma: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uover: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    uunder: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3or4IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType1or2IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class LimitSet(IdentifiedObject):
    """
    Specifies a set of Limits that are associated with a Measurement. A Measurement may have several LimitSets
      corresponding to seasonal or other changing conditions. The condition is captured in the name and description
      attributes. The same LimitSet may be used for several Measurements. In particular percentage limits are used
      this way.

    isPercentageLimits: Tells if the limit values are in percentage of normalValue or the specified Unit for
      Measurements and Controls.
    """

    isPercentageLimits: bool = Field(
        default=False,
        in_profiles=[
            Profile.OP,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.OP,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .EquipmentContainer import EquipmentContainer


@dataclass(config=DataclassConfig)
class Substation(EquipmentContainer):
    """
    A collection of equipment for purposes other than generation or utilization, through which electric energy in bulk
      is passed for the purposes of switching or modifying its characteristics.

    Region: The SubGeographicalRegion containing the substation.
    VoltageLevels: The voltage levels within this substation.
    DCConverterUnit: The DC converter unit belonging of the substation.
    """

    Region: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQBD,
            Profile.EQ,
        ],
    )

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # VoltageLevels : list = Field(default_factory=list, in_profiles = [Profile.EQBD, Profile.EQ, ])

    # *Association not used*
    # Type M:0..n in CIM  # pylint: disable-next=line-too-long
    # DCConverterUnit : list = Field(default_factory=list, in_profiles = [Profile.EQ, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQBD,
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindContPitchAngleIEC(IdentifiedObject):
    """
    Pitch angle control model. Reference: IEC 61400-27-1:2015, 5.6.5.2.

    dthetamax: Maximum pitch positive ramp rate (dthetamax) (> WindContPitchAngleIEC.dthetamin). It is a type-dependent
      parameter. Unit = degrees / s.
    dthetamin: Maximum pitch negative ramp rate (dthetamin) (< WindContPitchAngleIEC.dthetamax). It is a type-dependent
      parameter. Unit = degrees / s.
    kic: Power PI controller integration gain (KIc). It is a type-dependent parameter.
    kiomega: Speed PI controller integration gain (KIomega). It is a type-dependent parameter.
    kpc: Power PI controller proportional gain (KPc). It is a type-dependent parameter.
    kpomega: Speed PI controller proportional gain (KPomega). It is a type-dependent parameter.
    kpx: Pitch cross coupling gain (KPX). It is a type-dependent parameter.
    thetamax: Maximum pitch angle (thetamax) (> WindContPitchAngleIEC.thetamin). It is a type-dependent parameter.
    thetamin: Minimum pitch angle (thetamin) (< WindContPitchAngleIEC.thetamax). It is a type-dependent parameter.
    ttheta: Pitch time constant (ttheta) (>= 0). It is a type-dependent parameter.
    WindTurbineType3IEC: Wind turbine type 3 model with which this pitch control model is associated.
    """

    dthetamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dthetamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kic: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kiomega: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpomega: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpx: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetamin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ttheta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ACDCConverter import ACDCConverter


@dataclass(config=DataclassConfig)
class VsConverter(ACDCConverter):
    """
    DC side of the voltage source converter (VSC).

    CapabilityCurve: Capability curve of this converter.
    maxModulationIndex: The maximum quotient between the AC converter voltage (Uc) and DC voltage (Ud). A factor
      typically less than 1. It is converter`s configuration data used in power flow.
    delta: Angle between VsConverter.uv and ACDCConverter.uc. It is converter`s state variable used in power flow. The
      attribute shall be a positive value or zero.
    uv: Line-to-line voltage on the valve side of the converter transformer. It is converter`s state variable, result
      from power flow. The attribute shall be a positive value.
    droop: Droop constant. The pu value is obtained as D [kV/MW] x Sb / Ubdc. The attribute shall be a positive value.
    droopCompensation: Compensation constant. Used to compensate for voltage drop when controlling voltage at a distant
      bus. The attribute shall be a positive value.
    pPccControl: Kind of control of real power and/or DC voltage.
    qPccControl: Kind of reactive power control.
    qShare: Reactive power sharing factor among parallel converters on Uac control. The attribute shall be a positive
      value or zero.
    targetQpcc: Reactive power injection target in AC grid, at point of common coupling.  Load sign convention is used,
      i.e. positive sign means flow out from a node.
    targetUpcc: Voltage target in AC grid, at point of common coupling. The attribute shall be a positive value.
    targetPowerFactorPcc: Power factor target at the AC side, at point of common coupling. The attribute shall be a
      positive value.
    targetPhasePcc: Phase target at AC side, at point of common coupling. The attribute shall be a positive value.
    targetPWMfactor: Magnitude of pulse-modulation factor. The attribute shall be a positive value.
    VSCDynamics: Voltage source converter dynamics model used to describe dynamic behaviour of this converter.
    """

    CapabilityCurve: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    maxModulationIndex: float = Field(
        default=0.0,
        in_profiles=[
            Profile.EQ,
        ],
    )

    delta: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SV,
        ],
    )

    uv: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SV,
        ],
    )

    droop: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    droopCompensation: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    pPccControl: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SSH,
        ],
    )

    qPccControl: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.SSH,
        ],
    )

    qShare: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetQpcc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetUpcc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetPowerFactorPcc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetPhasePcc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    targetPWMfactor: float = Field(
        default=0.0,
        in_profiles=[
            Profile.SSH,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # VSCDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SV,
            Profile.SSH,
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class RemoteInputSignal(IdentifiedObject):
    """
    Supports connection to a terminal associated with a remote bus from which an input signal of a specific type is
      coming.

    Terminal: Remote terminal with which this input signal is associated.
    remoteSignalType: Type of input signal.
    DiscontinuousExcitationControlDynamics: Discontinuous excitation control model using this remote input signal.
    WindTurbineType1or2Dynamics: Wind generator type 1 or type 2 model using this remote input signal.
    PowerSystemStabilizerDynamics: Power system stabilizer model using this remote input signal.
    UnderexcitationLimiterDynamics: Underexcitation limiter model using this remote input signal.
    PFVArControllerType1Dynamics: Power factor or VAr controller type 1 model using this remote input signal.
    VoltageCompensatorDynamics: Voltage compensator model using this remote input signal.
    WindPlantDynamics: The wind plant using the remote signal.
    WindTurbineType3or4Dynamics: Wind turbine type 3 or type 4 models using this remote input signal.
    """

    Terminal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    remoteSignalType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    DiscontinuousExcitationControlDynamics: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType1or2Dynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    PowerSystemStabilizerDynamics: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    UnderexcitationLimiterDynamics: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    PFVArControllerType1Dynamics: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    VoltageCompensatorDynamics: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # WindPlantDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3or4Dynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcIEEEST3A(ExcitationSystemDynamics):
    """
    IEEE 421.5-2005 type ST3A model.  Some static systems utilize a field voltage control loop to linearize the exciter
      control characteristic. This also makes the output independent of supply source variations until supply
      limitations are reached.  These systems utilize a variety of controlled-rectifier designs: full thyristor
      complements or hybrid bridges in either series or shunt configurations. The power source can consist of only a
      potential source, either fed from the machine terminals or from internal windings. Some designs can have
      compound power sources utilizing both machine potential and current. These power sources are represented as
      phasor combinations of machine terminal current and voltage and are accommodated by suitable parameters in
      model type ST3A which is represented by ExcIEEEST3A. Reference: IEEE 421.5-2005, 7.3.

    vimax: Maximum voltage regulator input limit (VIMAX) (> 0).  Typical value = 0,2.
    vimin: Minimum voltage regulator input limit (VIMIN) (< 0).  Typical value = -0,2.
    ka: Voltage regulator gain (KA) (> 0). This is parameter K in the IEEE standard. Typical value = 200.
    ta: Voltage regulator time constant (TA) (>= 0).  Typical value = 0.
    tb: Voltage regulator time constant (TB) (>= 0).  Typical value = 10.
    tc: Voltage regulator time constant (TC) (>= 0).  Typical value = 1.
    vrmax: Maximum voltage regulator output (VRMAX) (> 0).  Typical value = 10.
    vrmin: Minimum voltage regulator output (VRMIN) (< 0).  Typical value = -10.
    km: Forward gain constant of the inner loop field regulator (KM) (> 0).  Typical value = 7,93.
    tm: Forward time constant of inner loop field regulator (TM) (> 0).  Typical value = 0,4.
    vmmax: Maximum inner loop output (VMMax) (> 0).  Typical value = 1.
    vmmin: Minimum inner loop output (VMMin) (<= 0).  Typical value = 0.
    kg: Feedback gain constant of the inner loop field regulator (KG) (>= 0).  Typical value = 1.
    kp: Potential circuit gain coefficient (KP) (> 0).  Typical value = 6,15.
    thetap: Potential circuit phase angle (thetap).  Typical value = 0.
    ki: Potential circuit gain coefficient (KI) (>= 0).  Typical value = 0.
    kc: Rectifier loading factor proportional to commutating reactance (KC) (>= 0). Typical value = 0,2.
    xl: Reactance associated with potential source (XL) (>= 0).  Typical value = 0,081.
    vbmax: Maximum excitation voltage (VBMax) (> 0).  Typical value = 6,9.
    vgmax: Maximum inner loop feedback voltage (VGMax) (>= 0).  Typical value = 5,8.
    """

    vimax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vimin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ka: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ta: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tb: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tc: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    km: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tm: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vmmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kg: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetap: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ki: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kc: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vbmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vgmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PhaseTapChangerNonLinear import PhaseTapChangerNonLinear


@dataclass(config=DataclassConfig)
class PhaseTapChangerSymmetrical(PhaseTapChangerNonLinear):
    """
    Describes a symmetrical phase shifting transformer tap model in which the voltage magnitude of both sides is the
      same. The difference voltage magnitude is the base in an equal-sided triangle where the sides corresponds to
      the primary and secondary voltages. The phase angle difference corresponds to the top angle and can be
      expressed as twice the arctangent of half the total difference voltage.

    """

    # No attributes defined for this class.

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
            Profile.SSH,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class TieFlow(IdentifiedObject):
    """
    Defines the structure (in terms of location and direction) of the net interchange constraint for a control area.
      This constraint may be used by either AGC or power flow.

    ControlArea: The control area of the tie flows.
    Terminal: The terminal to which this tie flow belongs.
    positiveFlowIn: Specifies the sign of the tie flow associated with a control area. True if positive flow into the
      terminal (load convention) is also positive flow into the control area.  See the description
      of ControlArea for further explanation of how TieFlow.positiveFlowIn is used.
    """

    ControlArea: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    Terminal: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.EQ,
        ],
    )

    positiveFlowIn: bool = Field(
        default=False,
        in_profiles=[
            Profile.EQ,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.EQ,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .TurbineGovernorDynamics import TurbineGovernorDynamics


@dataclass(config=DataclassConfig)
class GovSteamSGO(TurbineGovernorDynamics):
    """
    Simplified steam turbine governor.

    mwbase: Base for power values (MWbase) (> 0).  Unit = MW.
    t1: Controller lag (T1) (>= 0).
    t2: Controller lead compensation (T2) (>= 0).
    t3: Governor lag (T3) (> 0).
    t4: Delay due to steam inlet volumes associated with steam chest and inlet piping (T4) (>= 0).
    t5: Reheater delay including hot and cold leads (T5) (>= 0).
    t6: Delay due to IP-LP turbine, crossover pipes and LP end hoods (T6) (>= 0).
    k1: One / PU regulation (K1).
    k2: Fraction (K2).
    k3: Fraction (K3).
    pmax: Upper power limit (Pmax) (> GovSteamSGO.pmin).
    pmin: Lower power limit (Pmin) (>= 0 and < GovSteamSGO.pmax).
    """

    mwbase: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    k3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pmin: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .ExcitationSystemDynamics import ExcitationSystemDynamics


@dataclass(config=DataclassConfig)
class ExcELIN1(ExcitationSystemDynamics):
    """
    Static PI transformer fed excitation system ELIN (VATECH) - simplified model.  This model represents an all-static
      excitation system. A PI voltage controller establishes a desired field current set point for a proportional
      current controller. The integrator of the PI controller has a follow-up input to match its signal to the
      present field current.  A power system stabilizer with power input is included in the model.

    tfi: Current transducer time constant (Tfi) (>= 0).  Typical value = 0.
    tnu: Controller reset time constant (Tnu) (>= 0).  Typical value = 2.
    vpu: Voltage controller proportional gain (Vpu).  Typical value = 34,5.
    vpi: Current controller gain (Vpi).  Typical value = 12,45.
    vpnf: Controller follow up gain (Vpnf).  Typical value = 2.
    dpnf: Controller follow up deadband (Dpnf).  Typical value = 0.
    tsw: Stabilizer parameters (Tsw) (>= 0).  Typical value = 3.
    efmin: Minimum open circuit excitation voltage (Efmin) (< ExcELIN1.efmax).  Typical value = -5.
    efmax: Maximum open circuit excitation voltage (Efmax) (> ExcELIN1.efmin).  Typical value = 5.
    xe: Excitation transformer effective reactance (Xe) (>= 0).  Xe represents the regulation of the
      transformer/rectifier unit.  Typical value = 0,06.
    ks1: Stabilizer gain 1 (Ks1).  Typical value = 0.
    ks2: Stabilizer gain 2 (Ks2).  Typical value = 0.
    ts1: Stabilizer phase lag time constant (Ts1) (>= 0).  Typical value = 1.
    ts2: Stabilizer filter time constant (Ts2) (>= 0).  Typical value = 1.
    smax: Stabilizer limit output (smax).  Typical value = 0,1.
    """

    tfi: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tnu: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vpu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vpi: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vpnf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dpnf: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tsw: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    efmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    xe: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ts2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    smax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .PowerSystemStabilizerDynamics import PowerSystemStabilizerDynamics


@dataclass(config=DataclassConfig)
class Pss1A(PowerSystemStabilizerDynamics):
    """
    Single input power system stabilizer. It is a modified version in order to allow representation of various vendors'
      implementations on PSS type 1A.

    inputSignalType: Type of input signal (rotorAngularFrequencyDeviation, busFrequencyDeviation,
      generatorElectricalPower, generatorAcceleratingPower, busVoltage, or busVoltageDerivative).
    a1: Notch filter parameter (A1).
    a2: Notch filter parameter (A2).
    t1: Lead/lag time constant (T1) (>= 0).
    t2: Lead/lag time constant (T2) (>= 0).
    t3: Lead/lag time constant (T3) (>= 0).
    t4: Lead/lag time constant (T4) (>= 0).
    t5: Washout time constant (T5) (>= 0).
    t6: Transducer time constant (T6) (>= 0).
    ks: Stabilizer gain (Ks).
    vrmax: Maximum stabilizer output (Vrmax) (> Pss1A.vrmin).
    vrmin: Minimum stabilizer output (Vrmin) (< Pss1A.vrmax).
    vcu: Stabilizer input cutoff threshold (Vcu).
    vcl: Stabilizer input cutoff threshold (Vcl).
    a3: Notch filter parameter (A3).
    a4: Notch filter parameter (A4).
    a5: Notch filter parameter (A5).
    a6: Notch filter parameter (A6).
    a7: Notch filter parameter (A7).
    a8: Notch filter parameter (A8).
    kd: Selector (Kd).  true = e-sTdelay used false = e-sTdelay not used.
    tdelay: Time constant (Tdelay) (>= 0).
    """

    inputSignalType: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    a1: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a2: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t1: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t2: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t4: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t5: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    t6: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    ks: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vrmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vcu: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    vcl: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a3: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a4: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a5: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a6: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a7: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    a8: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kd: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    tdelay: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .RotatingMachineDynamics import RotatingMachineDynamics


@dataclass(config=DataclassConfig)
class AsynchronousMachineDynamics(RotatingMachineDynamics):
    """
    Asynchronous machine whose behaviour is described by reference to a standard model expressed in either time constant
      reactance form or equivalent circuit form or by definition of a user-defined model. Parameter details:
      Asynchronous machine parameters such as Xl, Xs, etc. are actually used as inductances in the model, but are
      commonly referred to as reactances since, at nominal frequency, the PU values are the same. However, some
      references use the symbol L instead of X.

    AsynchronousMachine: Asynchronous machine to which this asynchronous machine dynamics model applies.
    TurbineGovernorDynamics: Turbine-governor model associated with this asynchronous machine model.
    MechanicalLoadDynamics: Mechanical load model associated with this asynchronous machine model.
    WindTurbineType1or2Dynamics: Wind generator type 1 or type 2 model associated with this asynchronous machine model.
    """

    AsynchronousMachine: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # TurbineGovernorDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # MechanicalLoadDynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:0..1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType1or2Dynamics : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from typing import Optional
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from ..utils.base import Base


@dataclass(config=DataclassConfig)
class PositionPoint(Base):
    """
    Set of spatial coordinates that determine a point, defined in the coordinate system specified in
      'Location.CoordinateSystem'. Use a single position point instance to describe a point-oriented location. Use a
      sequence of position points to describe a line-oriented object (physical location of non-point oriented
      objects like cables or lines), or area of an object (like a substation or a geographical zone - in this case,
      have first and last position point with the same values).

    Location: Location described by this position point.
    sequenceNumber: Zero-relative sequence number of this point within a series of points.
    xPosition: X axis position.
    yPosition: Y axis position.
    zPosition: (if applicable) Z axis position.
    """

    Location: Optional[str] = Field(
        default=None,
        in_profiles=[
            Profile.GL,
        ],
    )

    sequenceNumber: int = Field(
        default=0,
        in_profiles=[
            Profile.GL,
        ],
    )

    xPosition: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    yPosition: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    zPosition: str = Field(
        default="",
        in_profiles=[
            Profile.GL,
        ],
    )

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.GL,
        }



# SPDX-FileCopyrightText: 2023 Alliander
#
# SPDX-License-Identifier: Apache-2.0

"""
Generated from the CGMES 3 files via cimgen: https://github.com/sogno-platform/cimgen
"""

from functools import cached_property
from pydantic import Field
from pydantic.dataclasses import dataclass
from ..utils.dataclassconfig import DataclassConfig
from ..utils.profile import BaseProfile, Profile

from .IdentifiedObject import IdentifiedObject


@dataclass(config=DataclassConfig)
class WindContPType3IEC(IdentifiedObject):
    """
    P control model type 3. Reference: IEC 61400-27-1:2015, 5.6.5.4.

    dpmax: Maximum wind turbine power ramp rate (dpmax). It is a type-dependent parameter.
    dprefmax: Maximum ramp rate of wind turbine reference power (dprefmax). It is a project-dependent parameter.
    dprefmin: Minimum ramp rate of wind turbine reference power (dprefmin). It is a project-dependent parameter.
    dthetamax: Ramp limitation of torque, required in some grid codes (dtmax). It is a project-dependent parameter.
    dthetamaxuvrt: Limitation of torque rise rate during UVRT (dthetamaxUVRT). It is a project-dependent parameter.
    kdtd: Gain for active drive train damping (KDTD). It is a type-dependent parameter.
    kip: PI controller integration parameter (KIp). It is a type-dependent parameter.
    kpp: PI controller proportional gain (KPp). It is a type-dependent parameter.
    mpuvrt: Enable UVRT power control mode (MpUVRT).  It is a project-dependent parameter. true = voltage control (1 in
      the IEC model) false = reactive power control (0 in the IEC model).
    omegaoffset: Offset to reference value that limits controller action during rotor speed changes (omegaoffset). It is
      a case-dependent parameter.
    pdtdmax: Maximum active drive train damping power (pDTDmax). It is a type-dependent parameter.
    tdvs: Time delay after deep voltage sags (TDVS) (>= 0). It is a project-dependent parameter.
    thetaemin: Minimum electrical generator torque (temin). It is a type-dependent parameter.
    thetauscale: Voltage scaling factor of reset-torque (tuscale). It is a project-dependent parameter.
    tomegafiltp3: Filter time constant for generator speed measurement (Tomegafiltp3) (>= 0). It is a type-dependent
      parameter.
    tpfiltp3: Filter time constant for power measurement (Tpfiltp3) (>= 0). It is a type-dependent parameter.
    tpord: Time constant in power order lag (Tpord). It is a type-dependent parameter.
    tufiltp3: Filter time constant for voltage measurement (Tufiltp3) (>= 0). It is a type-dependent parameter.
    tomegaref: Time constant in speed reference filter (Tomega,ref) (>= 0). It is a type-dependent parameter.
    udvs: Voltage limit for hold UVRT status after deep voltage sags (uDVS). It is a project-dependent parameter.
    updip: Voltage dip threshold for P-control (uPdip).  Part of turbine control, often different (e.g 0.8) from
      converter thresholds. It is a project-dependent parameter.
    omegadtd: Active drive train damping frequency (omegaDTD). It can be calculated from two mass model parameters. It
      is a type-dependent parameter.
    zeta: Coefficient for active drive train damping (zeta). It is a type-dependent parameter.
    WindTurbineType3IEC: Wind turbine type 3 model with which this wind control P type 3 model is associated.
    WindDynamicsLookupTable: The wind dynamics lookup table associated with this P control type 3 model.
    """

    dpmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dprefmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dprefmin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dthetamax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    dthetamaxuvrt: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kdtd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kip: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    kpp: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    mpuvrt: bool = Field(
        default=False,
        in_profiles=[
            Profile.DY,
        ],
    )

    omegaoffset: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    pdtdmax: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tdvs: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetaemin: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    thetauscale: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tomegafiltp3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpfiltp3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tpord: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tufiltp3: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    tomegaref: int = Field(
        default=0,
        in_profiles=[
            Profile.DY,
        ],
    )

    udvs: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    updip: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    omegadtd: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    zeta: float = Field(
        default=0.0,
        in_profiles=[
            Profile.DY,
        ],
    )

    # *Association not used*
    # Type M:1 in CIM  # pylint: disable-next=line-too-long
    # WindTurbineType3IEC : Optional[str] = Field(default=None, in_profiles = [Profile.DY, ])

    # *Association not used*
    # Type M:1..n in CIM  # pylint: disable-next=line-too-long
    # WindDynamicsLookupTable : list = Field(default_factory=list, in_profiles = [Profile.DY, ])

    @cached_property
    def possible_profiles(self) -> set[BaseProfile]:
        """
        A resource can be used by multiple profiles. This is the set of profiles
        where this element can be found.
        """
        return {
            Profile.DY,
        }
